library cork.src.injector;

import 'dart:collection';

/// A simple injector that returns an instance by type.
class Injector {
  final Map<Type, TypeFactory> _typeFactories;
  final _typeInstances = new HashMap<Type, Object>();

  /// Create a new injector with a map of types that can be created.
  Injector([this._typeFactories = const {}]);

  /// Returns, creating if necessary, an instance of [type]. If [type] was
  /// previously created, the last instance is returned.
  Object get(Type type) {
    var instance = _typeInstances[type];
    if (instance == null) {
      final factory = _typeFactories[type];
      if (factory == null) {
        throw new ArgumentError('No factory found for $type.');
      }
      final arguments = factory.parameterTypes.map(get).toList(growable: false);
      _typeInstances[type] = instance = factory.call(arguments);
    }
    return instance;
  }
}

/// A decorated constructor that can be easily consumed by an injector.
abstract class TypeFactory<T> implements Function {
  /// Creates a new instance.
  T call([List positionalArguments]);

  /// A collection of all parameter types of this factory.
  Iterable<Type> get parameterTypes;
}
